(* set_test.ml: demonstrate how use of polymorphic comparison can slow
   does set searches more than using a comparison specific to a type,
   in this case pairs of strings. *)

open Printf;;

type strpair = {
    first : string;
    second : string;
};;

module PolyCmp = struct
  type t = strpair;;
  let compare = Pervasives.compare;;                          (* polymorphic comparison *)
end;;

module StringCmp = struct
  type t = strpair;;
  let compare a b =                                           (* specific comparison *)
    let diff = String.compare a.first b.first in
    if diff=0 then
      String.compare a.second b.second
    else
      diff
  ;;
end;;

module PolySet    = Set.Make(PolyCmp);;
module StringSet  = Set.Make(StringCmp);;

let _ =
  let narg = Array.length Sys.argv in
  if narg < 2 then
    (printf "usage: %s <int>\n" Sys.argv.(0);
     exit 1);

  let polyset = ref PolySet.empty in
  let stringset =  ref StringSet.empty in

  let len = int_of_string Sys.argv.(1) in
  let nums = Array.make len {first="";second=""} in              
  Random.init 123456789;
  for i=0 to len-1 do
    let first  = string_of_int (Random.int 10000) in
    let second = string_of_int (Random.int 10000) in
    nums.(i) <- {first;second};
    let first  = string_of_int (Random.int 10000) in
    let second = string_of_int (Random.int 10000) in
    polyset   := PolySet.add   {first;second} !polyset;
    stringset := StringSet.add {first;second} !stringset;
  done;

  let polyset = !polyset in
  let stringset  = !stringset in

  printf "polyset search\n";
  let count = ref 0 in
  let start_time = Sys.time () in
  for i=0 to len-1 do
    if PolySet.mem nums.(i) polyset then
      incr count;
  done;
  let stop_time = Sys.time () in
  let tot_time = stop_time -. start_time in
  printf "count: %d,  time: %.4f secs\n" !count tot_time;
  
  printf "stringset search\n";
  let count = ref 0 in
  let start_time = Sys.time () in
  for i=0 to len-1 do
    if StringSet.mem nums.(i) stringset then
      incr count;
  done;
  let stop_time = Sys.time () in
  let tot_time = stop_time -. start_time in
  printf "count: %d,  time: %.4f secs\n" !count tot_time;
;;
